/*****************************************************************************
 * $CAMITK_LICENCE_BEGIN$
 *
 * CamiTK - Computer Assisted Medical Intervention ToolKit
 * (c) 2001-2013 UJF-Grenoble 1, CNRS, TIMC-IMAG UMR 5525 (GMCAO)
 *
 * Visit http://camitk.imag.fr for more information
 *
 * This file is part of CamiTK.
 *
 * CamiTK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * CamiTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with CamiTK.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $CAMITK_LICENCE_END$
 ****************************************************************************/

// -- Qt dependencies
#include <QFileDialog>
#include <QMenuBar>
#include <QMessageBox>
#include <QTextStream>
#include <QString>
#include <QDate>

// -- Qt State Machine stuff
#include <QFinalState>

// -- CamiTK Core stuff
#include <MainWindow.h>
#include <Explorer.h>
#include <Application.h>
#include <MedicalImageViewer.h>
#include <Core.h>
#include <ExtensionManager.h>
#include <Explorer.h>
#include <Log.h>


// -- CamiTK Application Local stuff
#include "ActionStateMachine.h"
#include "ActionStateViewer.h"
#include "ActionTransition.h"


// constructor
ActionStateMachine::ActionStateMachine(int &argc, char ** argv) throw(AbortException) : 
Application("Action State Machine Main Window", argc, argv) 
{
	statesMap = new QMap<QString, ActionState *>();

	// Atributes initialization
	mainWindow = NULL;
	name = QString("Action State Machine");


	// CamiTK Application main window
	initMainWindow();

	// Load File
	QString filename;
	if (argc < 2)
	{
		filename = QFileDialog::getOpenFileName( NULL, "Please select an xml file");
	}
	else {
		filename = argv[1];
	}

		
	// check the given file
	this->checkSCXMLFile(filename);

	// parse XML file and store actions and transitions in states
	name = this->parseSCXMLTree();
	mainWindow->setWindowSubtitle(name);

	// Load exra components

	// Load extra actions



	// Prepare the output directory
	QFile::copy(":/resources/log2html.xsl", saveDirectory.absolutePath() + "/log2html.xsl");
	

	// Starting the Machine !!

	//create the main timer
	QDate date = QDate::currentDate();
	startTime = new QTime();
	startTime->start();

	(*logStream) << "<?xml version='1.0' encoding='UTF-8' ?>" << endl;
	(*logStream) << "<?xml-stylesheet type='text/xsl' href='log2html.xsl'?>" << endl << endl;
	(*logStream) << "<application xmlns='http://ujf-grenoble.fr/camitk/logApplication'>" << endl;
	(*logStream) << "	<name>" << name << "</name>" << endl;
	(*logStream) << "	<startDate>" << date.toString("yyyy-MM-dd") << "</startDate>" << endl;
	(*logStream) << "	<startTime>" << startTime->toString("hh:mm:ss:zzz") << "</startTime>" << endl;

    machine.start();	


}

// destructor
ActionStateMachine::~ActionStateMachine()
{
	QTime endTime = QTime::currentTime();

	(*logStream) << "	<endTime>" << endTime.toString("hh:mm:ss:zzz") << "</endTime>" << endl;
	(*logStream) << "	<timeEnlapsed unit='ms'>" << (startTime->elapsed()) << "</timeEnlapsed>" << endl;
	(*logStream) << "</application>" << endl;
	logFile->close();
}

void ActionStateMachine::initMainWindow() {

	mainWindow = new MainWindow(name);
	mainWindow->setCentralViewer(MedicalImageViewer::getInstance());
	mainWindow->addDockViewer(Qt::RightDockWidgetArea, ActionStateViewer::getActionStateViewer());
    Explorer * explorer = new Explorer();
    mainWindow->addDockViewer(Qt::LeftDockWidgetArea, explorer);


	mainWindow->showStatusBar(true);
	this->setMainWindow(mainWindow);

}

void ActionStateMachine::checkSCXMLFile(QString filename) throw (AbortException) {
	QString msg;
	QDomDocument doc;

	//read the file 
	QFile file(filename);
	if (!file.open(QIODevice::ReadOnly)) {
		msg = "File not found: the file " + filename + " was not found";
		CAMITK_ERROR("ActionStateMachine", "setFileName", msg.toStdString());
		msg = "Error: cannot fine file " + filename + " sorry...\n";
		throw AbortException (msg.toStdString()); 
	}
	if (!doc.setContent(&file)) {
		msg = "File " + filename + " have no valid root (not a valid XML file format).\n";
		file.close();
		CAMITK_ERROR("ActionStateMachine", "setFileName", msg.toStdString());
		throw AbortException(msg.toStdString());
	}
	
	QString rootName = doc.documentElement().nodeName();
	if (rootName != QString("scxml")) {
		file.close();
		msg = "File " + filename + " is not in the right scxml file format.\n";
		CAMITK_ERROR("ActionStateMachineWindow", "setFileName", msg.toStdString());
		throw AbortException(msg.toStdString());
	}

	// Ok, after all this checking, the file seems to be good looking,
	// set it as the right xml doc...
	this->scxmlDoc = doc;
	
	file.close();
}

QString ActionStateMachine::parseSCXMLTree() throw(AbortException) {
	// Get the name of the application
	QString applicationName = "";
	QDomElement docElem = scxmlDoc.documentElement();
	applicationName = docElem.attribute("camitk:applicationName");

	// Get the name of the saving directory
	QString saveDir = docElem.attribute("camitk:saveDirectory");
	if (saveDir.isEmpty()) {
		saveDirectory = QFileDialog::getExistingDirectory(this->mainWindow, "Please select a directory where to save log and component files");
	}
	else {
		saveDirectory = QDir(saveDir);
	}
	QDate date = QDate::currentDate();
	QTime time = QTime::currentTime();
	QString newDir = QString("%1-%2-%3at%4h%5").arg(date.year())
										 .arg(date.month())
										 .arg(date.day())
										 .arg(time.hour())
										 .arg(time.minute());

	if (! saveDirectory.mkdir(newDir))  {
		saveDirectory = QFileDialog::getExistingDirectory(this->mainWindow, "Please select a directory where to save log and component files");
		saveDirectory.mkdir(newDir);
	}
	saveDirectory.cd(newDir);

	QString logFileName = this->saveDirectory.absolutePath() + "/log.xml";
	logFile = new QFile(logFileName);
    logFile->open(QIODevice::WriteOnly | QIODevice::Text);
	logStream = new QTextStream(logFile);


	// Get the states node
	QDomNodeList theStates = scxmlDoc.elementsByTagName("state");
	
	// Create ActionStates
	createAllActionStates(theStates);
	
	// Now that all the states are created and stored in the map, 
	// create the transitions in between.
	createTransitions(theStates);

	// Set the initial state (specified in the scxml element with initial attribute).
	QString initialStateName = scxmlDoc.documentElement().attribute("initial");
	ActionState * initialState = statesMap->find(initialStateName).value();
    machine.setInitialState(initialState);
	ActionStateViewer::getActionStateViewer()->setState(initialState);

	// Connect the end of the machine with the end of the application
	QObject::connect(&machine, SIGNAL(finished()), QApplication::instance(), SLOT(quit()));

	// Ok, Actions and Wizard are created. We do not need the file any more.
	// Clean up:
	delete statesMap;

	return applicationName;
}


void ActionStateMachine::createAllActionStates(QDomNodeList nodeList) throw(AbortException) {
	int nbStates = nodeList.size();
	if (nbStates < 1) {
		QString msg = "No state and no camitk-action ! \n";
		CAMITK_ERROR("ActionStateMachine", "parseSCXMLTree", msg.toStdString());		
		throw AbortException (msg.toStdString());
	}

	for (int nb = 0; nb < nbStates; nb++) {
		QDomElement element = nodeList.item(nb).toElement();
		if (! element.isNull()) {
			QString elementName = element.nodeName();
			if (elementName == QString("state")) {
				QString stateId = element.attribute("id");
				QString stateDescription = element.elementsByTagName("camitk:description").item(0).toElement().text();
				ActionState * state = new ActionState(&machine, stateId, stateDescription, logStream);
				if (element.elementsByTagName("camitk:action").size()  > 0) {
					QDomElement actionElement = element.elementsByTagName("camitk:action").item(0).toElement();
					// Find the name of the action
					QString camiTKActionName = actionElement.elementsByTagName("camitk:name").item(0).toElement().text();
					if (camiTKActionName == "Save") {
						delete(state);
						state = new SaveActionState(&machine, stateId, stateDescription, saveDirectory.absolutePath(), logStream);
						setCamiTKSaveAction((SaveActionState *) state, actionElement);
					}
					else {
						setCamiTKAction(state, actionElement);
					}
				}
				QDomNodeList alternativeDescriptions = element.elementsByTagName("camitk:altDescription");
				for (int i = 0; i < alternativeDescriptions.size(); i++) {
					QDomElement altDescElmt = alternativeDescriptions.item(i).toElement();
					QString altDescText = altDescElmt.text();
					QString altDescCond =  altDescElmt.attribute("camitk:previousStatus", "");
					QVector<Action::ApplyStatus> statusList = stringToStatus(altDescCond);

					state->setAleternativeDesc(altDescText, statusList);

				}
				//if (element.attribute("final", "false").toLower() == "true") {
				//	//state->setFinal();

				//}

				statesMap->insert(stateId, state);
			}
		}
	}
	// Do not forget the final state
	theEnd = new QFinalState(&machine);
}

void ActionStateMachine::createTransitions(QDomNodeList nodeList) throw(AbortException)  {
	int nbStates = nodeList.size();
	for (int stateNumber = 0; stateNumber < nbStates; stateNumber++) {
		// For each state ...
		QDomElement element = nodeList.item(stateNumber).toElement();

		// Find its name (id attribute)...
		QString stateId = element.attribute("id");
		// ... and get the corresponding created state in the map:
		ActionState * state = (statesMap->find(stateId)).value();
		// Check if this state is final
		bool isFinal = (element.attribute("final", "false").toLower() == QString("true"));


		// Find its transitions (transition children nodes)
		QDomNodeList transitionsList = element.elementsByTagName("transition");
		for (int transNb = 0; transNb < transitionsList.size(); transNb++) {
			QDomElement transElement = transitionsList.item(transNb).toElement();
			if (! transElement.isNull()) {
				QString eventName  = transElement.attribute("event");
				QString targetName = transElement.attribute("target");

				// Apply the previous action during this transition ?
				QString applyActionStr = transElement.attribute("camitk:applyAction", "true");
				bool applyAction = (applyActionStr.toLower() == "true");

				// Find disable conditions
				QString disableConds = transElement.attribute("camitk:disable", "");
				QVector<Action::ApplyStatus> disableConditions = stringToStatus(disableConds);

				// Find the corresponding target state
				ActionState * target = (statesMap->find(targetName)).value();

				// Create the actual transition:
				ActionTransition * trans = 
					state->addActionTransition(eventName, target, applyAction, disableConditions);

				// Check if the transition should close some components...
				QDomNodeList toClose = transElement.elementsByTagName("camitk:close");
				if (toClose.size() > 0) {
					QDomNodeList compsToClose = toClose.item(0).toElement().elementsByTagName("camitk:component");
					for (int idx = 0; idx < compsToClose.size(); idx++) {
						QDomElement oneComp = compsToClose.item(idx).toElement();
						QString compName = oneComp.attribute("name");
						QString compType = oneComp.attribute("type");
						trans->addComponentToClose(compName, compType);
					}
				}
			}
		}

		if (isFinal) {
			state->addActionTransition("Quit", theEnd, false);
		}
	}
	
}

QVector<Action::ApplyStatus> ActionStateMachine::stringToStatus(QString listOfStatus) {
	QVector<Action::ApplyStatus> statusList;
	statusList.clear();

	QStringList statusStringList = listOfStatus.split(", ");
	for (QStringList::iterator it = statusStringList.begin(); it != statusStringList.end(); it++) {
		if((*it) == "SUCCESS") {
			statusList.append(Action::SUCCESS);
		} 
		else if ((*it) == "ERROR") {
			statusList.append(Action::ERROR);
		}
		else if ((*it) == "WARNING") {
			statusList.append(Action::WARNING);
		}
		else if ((*it) == "ABORTED") {
			statusList.append(Action::ABORTED);
		}
		else if ((*it) == "TRIGGERED") {
			statusList.append(Action::TRIGGERED);
		}
	}

	return statusList;
}


void ActionStateMachine::setCamiTKAction(ActionState * actionState, QDomElement actionElement) throw (AbortException) {
	QString msg;
	if ((actionState == NULL) || (actionElement.isNull())) {
		msg = " No Action or no state...";
		CAMITK_ERROR("ActionStateMachine", "setCamiTKAction", msg.toStdString());
		throw AbortException (msg.toStdString()); 		
	}
	
	QString actionName = actionElement.elementsByTagName("camitk:name").item(0).toElement().text();
	Action * action = Application::getAction(actionName);
	if (action == NULL) {
		msg = "Action " + actionName + " is not a proper CamiTK action.";
		CAMITK_ERROR("ActionStateMachine", "setCamiTKAction", msg.toStdString());
		throw AbortException(msg.toStdString());
	}

	// Parameters
	QMap<QString, QVariant> parameters;
	parameters.clear();
	QDomNodeList domParameters = actionElement.elementsByTagName("camitk:parameters");
	if (! domParameters.isEmpty()) {
		QDomNodeList parameterList = domParameters.item(0).toElement().elementsByTagName("camitk:parameter");
		for (int i = 0; i < parameterList.size(); i++) {
			QDomElement param  = parameterList.item(i).toElement();
			QString paramName  = param.attribute("name");
			QString paramType  = param.attribute("type");
			QString paramValue = param.attribute("value");
			QVariant val;

			QStringList possibleTypes;
			possibleTypes << "int" << "bool" << "double" << "QString";


			switch(possibleTypes.indexOf(paramType)) {
				case 0: // int
					val = QVariant(paramValue.toInt());
					break;
				case 1: // bool
					val = QVariant(paramValue);
					break;
				case 2: // double
					val = QVariant(paramValue.toDouble());
					break;
				case 3: // QString
					val = QVariant(paramValue);
					break;
				default:
					val = QVariant(paramValue);
					break;
			}
			parameters.insert(paramName, val);
		}
	}

	// Input Component Names
	QMap<QString, QString> inputComponentNames;
	inputComponentNames.clear();
	QDomNodeList inputList = actionElement.elementsByTagName("camitk:inputs");
	if (! inputList.isEmpty()) {
		QDomNodeList inputComps = inputList.item(0).toElement().elementsByTagName("camitk:component");
		for (int i = 0; i < inputComps.size(); i++) {
			QDomElement inputComp = inputComps.item(i).toElement();
			QString compName = inputComp.attribute("name");
			QString compType = inputComp.attribute("type");
			inputComponentNames.insert(compName, compType);
		}
	}

	// Output Component Names
	QMap<QString, QString> outputComponentNames;
	outputComponentNames.clear();
	QDomNodeList outputList = actionElement.elementsByTagName("camitk:outputs");
	if (! outputList.isEmpty()) {
		QDomNodeList outputComps = outputList.item(0).toElement().elementsByTagName("camitk:component");
		for (int i = 0; i < outputComps.size(); i++) {
			QDomElement outputComp = outputComps.item(i).toElement();
			QString compName = outputComp.attribute("name");
			QString compType = outputComp.attribute("type");
			outputComponentNames.insert(compName, compType);
		}
	}

	actionState->setAction(action, parameters, inputComponentNames, outputComponentNames);



}

void ActionStateMachine::setCamiTKSaveAction(SaveActionState * actionState, QDomElement actionElement) throw (AbortException) {

	QString msg;
	if ((actionState == NULL) || (actionElement.isNull())) {
		msg = " No Action or no state...";
		CAMITK_ERROR("ActionStateMachine", "setCamiTKSaveAction", msg.toStdString());
		throw AbortException (msg.toStdString()); 		
	}
	

	// Input Component Names, Types, Extensions and Directories....
	QVector<SaveActionState::saveComponentsInfo> inputInfos;
	QDomNodeList inputList = actionElement.elementsByTagName("camitk:inputs");
	if (! inputList.isEmpty()) {
		QDomNodeList inputComps = inputList.item(0).toElement().elementsByTagName("camitk:component");
		for (int i = 0; i < inputComps.size(); i++) {
			QDomElement inputComp = inputComps.item(i).toElement();
			SaveActionState::saveComponentsInfo compInfos;
			compInfos.name = inputComp.attribute("name");
			compInfos.type = inputComp.attribute("type");
			compInfos.extension = inputComp.attribute("suffix");
			compInfos.directory = inputComp.attribute("saveDirectory");

			inputInfos.append(compInfos);
		}
	}

	actionState->setInput(inputInfos);
}


QString ActionStateMachine::getSaveDirectory() {
	return this->saveDirectory.absolutePath();
}
